#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import datetime
import matplotlib.gridspec as gridspec
import icesm
import plotting
import util
import copy
import matplotlib.colors as mcolors

#------------------------------------------------------------------------------#
# 'namelist'
plotvars = ['dexc_sp']

timeind =slice(None)
levind = slice(None)
latind = slice(None)
lonind = slice(None)

startdate = datetime.datetime(2001, 1, 1)
stopdate = datetime.datetime(2005, 12, 31)

datelist_files = util.monthlist(startdate, stopdate)

runnames_list = [['10_Si_param_SAT_TMP373_RVP'],
                 ['LGM_Si_param_SAT_TMP373_RVP'],
                 ['10_Si_param_SAT_TMP373_RVP',
                  'LGM_Si_param_SAT_TMP373_RVP'],
                 ['10_Si_real_avg_SAT_TMP373_RVP_new'],
                 ['LGM_Si_real_avg_SAT_TMP373_RVP'],
                 ['10_Si_real_avg_SAT_TMP373_RVP_new',
                  'LGM_Si_real_avg_SAT_TMP373_RVP'],
                 ['10_Si_param_SAT_TMP373_RVP',
                  '10_Si_real_avg_SAT_TMP373_RVP_new'],
                 ['LGM_Si_param_SAT_TMP373_RVP',
                  'LGM_Si_real_avg_SAT_TMP373_RVP']]

runtitles = {}
runtitles['10_Si_param_SAT_TMP373_RVP'] = '$Si_{param}$'
runtitles['10_Si_real_avg_SAT_TMP373_RVP_new'] = '$Si_{real}$'
runtitles['LGM_Si_param_SAT_TMP373_RVP'] = '$Si_{param}$'
runtitles['LGM_Si_real_avg_SAT_TMP373_RVP'] = '$Si_{real}$'

climtitles = {}
for key in runtitles.keys():
    climtitles[key] = 'LGM' if 'LGM' in key else 'PD'

letters = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']

qvars = ['sp']
isos = ['H2O', 'HDO', 'H218O']
loadvars = icesm.combine_isoloadvars(qvars, isos, ['LANDFRAC'])

#------------------------------------------------------------------------------#
# load
data = {}
data_avg = {}
data_diff = {}
for runname in set(util.flatten(runnames_list)):
    if 'LGM' in runname:
        datelist_files2 = [datetime.datetime(date.year-150, date.month,
                           date.day) for date in datelist_files]
    else:
        datelist_files2 = copy.deepcopy(datelist_files)
    filepaths = ['../model/' + runname+ '/' + runname + '.cam.h0.' + \
                 date.strftime('%Y-%m') +'.nc' \
                 for date in datelist_files2]

    data[runname] = icesm.ncopen(filepaths,
                                 varnames=loadvars,
                                 ind=[timeind, levind, latind, lonind])

    data_avg[runname] = {}
    for var in data[runname].keys():
        if data[runname][var].ndim >= 3:
            data_avg[runname][var] = np.mean(data[runname][var],axis=0)
        else:
            data_avg[runname][var] = copy.deepcopy(data[runname][var])

    icesm.swap_lons(data_avg[runname])
    icesm.deltas(data_avg[runname], qvars=qvars)

    data_avg[runname]['lons'], data_avg[runname]['lats'] = \
        np.meshgrid(data_avg[runname]['lon'], data_avg[runname]['lat'])

data_diff = {}
for runnames in runnames_list:
    data_diff['-'.join(runnames[::-1])] = {}
    for var in data_avg[runnames[0]].keys():
        if len(runnames) > 1: # and var not in ['U', 'V']:
            data_diff['-'.join(runnames[::-1])][var] = \
                data_avg[runnames[1]][var] - data_avg[runnames[0]][var]
        else:
            data_diff[runnames[0]][var] = copy.deepcopy(data_avg[runnames[0]][var])

# global and antarctic averages
avg = {}
global_ind = data_avg[runnames_list[0][0]]['lats'] > -999
antarctica_ind = (data_avg[runnames_list[0][0]]['lats'] < -60.) & \
                 (data_avg[runnames_list[0][0]]['LANDFRAC'] > 0.5)
for runnames in runnames_list:
    avg['-'.join(runnames[::-1])] = {}
    for plotvar in plotvars:
        avg['-'.join(runnames[::-1])][plotvar] = np.zeros(2)
        for i,ind in enumerate([global_ind, antarctica_ind]):
            ind[:,-1] = False #because lon 180 appears 2x (for plotting)
            avg['-'.join(runnames[::-1])][plotvar][i] = \
                np.mean(data_diff['-'.join(runnames[::-1])][plotvar][ind] * \
                np.cos(data_avg[runnames[0]]['lats'][ind]/180.*np.pi)) / \
                np.mean(np.cos(data_avg[runnames[0]]['lats'][ind]/180.*np.pi))

#------------------------------------------------------------------------------#
# measurements

titlelist = ['site', 'lat', 'lon', 'dD_sp', 'd18O_sp', 'dexc_sp']
formatlist = ['U10', float, float, float, float, float]
data_avg['icecores_lgm'] = np.genfromtxt('icecores_LGM.txt',
    dtype={'names':titlelist,'formats':formatlist}, skip_header=1)
data_avg['icecores_pd'] = np.genfromtxt('icecores_PD.txt',
    dtype={'names':titlelist,'formats':formatlist}, skip_header=1)

#------------------------------------------------------------------------------#
for runnames in runnames_list:
    data_diff['icecores_'+'-'.join(runnames[::-1])] = {}
    for plotvar in [var for var in plotvars if var in \
        data_avg['icecores_lgm'].dtype.names] + ['lat', 'lon']:
        data_diff['icecores_'+'-'.join(runnames[::-1])][plotvar] = \
            np.zeros(len(data_avg['icecores_lgm']['site']))

for s,site in enumerate(data_avg['icecores_lgm']['site']):
    sind = (data_avg['icecores_lgm']['site'] == site)

    for runnames in runnames_list:
        for plotvar in [var for var in plotvars if var in \
            data_avg['icecores_lgm'].dtype.names]:
            if len(runnames) == 1:
                if 'LGM' in runnames[0]:
                    data_diff['icecores_'+runnames[0]][plotvar][s] = \
                        data_avg['icecores_lgm'][plotvar][sind]
                else:
                    data_diff['icecores_'+runnames[0]][plotvar][s] = \
                        data_avg['icecores_pd'][plotvar][sind]
            elif len(runnames) == 2:
                if 'LGM' in runnames[1] and not 'LGM' in runnames[0]:
                    data_diff['icecores_'+'-'.join(runnames[::-1])][plotvar][s] = \
                        data_avg['icecores_lgm'][plotvar][sind] - \
                        data_avg['icecores_pd'][plotvar][sind]
        for coord in ['lat', 'lon']:
            data_diff['icecores_'+'-'.join(runnames[::-1])][coord][s] = \
                data_avg['icecores_lgm'][coord][sind]

#------------------------------------------------------------------------------#
# contour levels
scale = {'d18O_sp'    : 18.75,
         'dD_sp'      : 150,
         'dexc_sp'    : 10}

levdict = {}
for var in scale.keys():
    levdict[var] = np.arange(-2.0,2.05,0.1)*scale[var]

# map
m = Basemap(projection='nsper', satellite_height=1e7,
            lon_0=0, lat_0=-90, resolution='c')

#------------------------------------------------------------------------------#
# plot

fontsize = 23

plotting.setup_plot(fontsize=fontsize)
pd = plotting.make_plotdict()

for plotvar in plotvars:
    plt.close()
    fig = plt.figure()
    fig.set_size_inches(15, 0.5+4*np.ceil(len(runnames_list)/3), forward=True)
    gs = gridspec.GridSpec(4*int(np.ceil(len(runnames_list)/3))+3, 4*3+1,
                           height_ratios=[1.0]+[7.0,-1.0,0.79,0.0]*\
                           int(np.ceil(len(runnames_list)/3))+[0.2, 0.75],
                           width_ratios=[1.0,7.0,-2.0,1.0]*3+[0.75])

    for r,runnames in enumerate(runnames_list):
        ax = fig.add_subplot(gs[4*int(r/3)+1,4*(r%3):4*(r%3)+3])

        m.drawcoastlines(linewidth=0.5)
        m.drawmeridians(np.arange(0,360,30), linewidth=0.15)
        m.drawparallels(np.arange(-90,90,10), linewidth=0.15)

        h1 = m.pcolor(data_avg[runnames[0]]['lons'],
                      data_avg[runnames[0]]['lats'],
                      np.squeeze(data_diff['-'.join(runnames[::-1])][plotvar]),
                      cmap=pd[plotvar]['cmap'],
                      vmin=levdict[plotvar][0], vmax=levdict[plotvar][-1],
                      latlon=True)

        if not 'precipitation' in pd[plotvar]['add'] and not 'surface' in \
                pd[plotvar]['longname'] and 1==2:
            plt.text(0.005,0.01,'lev='+str(30-levind),
                     transform=ax.transAxes, horizontalalignment='left',
                     verticalalignment='bottom', fontsize=fontsize)

        # ice cores
        if (r+1)/3 <= 2 and 1==1:
            h2 = m.scatter(data_diff['icecores_'+'-'.join(runnames[::-1])]['lon'],
                           data_diff['icecores_'+'-'.join(runnames[::-1])]['lat'],
                           c=data_diff['icecores_'+'-'.join(
                               runnames[::-1])][plotvar],
                           s=80, latlon=True, cmap=pd[plotvar]['cmap'],
                           vmin=levdict[plotvar][0], vmax=levdict[plotvar][-1],
                           edgecolor='black', linewidth=0.5)

            for s,site in enumerate(data_avg['icecores_lgm']['site']):
                x, y = m(data_diff['icecores_'+'-'.join(runnames[::-1])]['lon'][s],
                         data_diff['icecores_'+'-'.join(runnames[::-1])]['lat'][s])
                if site == 'WD':
                    x -= 1.2e6
                    y -= 0.7e6
                elif site == 'TAL':
                    y -= 0.7e6
                ax.annotate(site, (x+1e5, y+1e5), fontsize=fontsize-10)

        if 4*(r%3) == 0 and len(runnames_list)>3:
            plt.text(-0.20, 0.5, ' -- '.join([runtitles[runname] for runname in \
                runnames[::-1]]), transform=ax.transAxes,
                horizontalalignment='right', verticalalignment='center',
                fontsize=fontsize+2, rotation=90)
        if int(r/3) == 0: #!!!!!!
            plt.text(0.5, 1.15, ' -- '.join([climtitles[runname] for runname in \
                runnames[::-1]]), transform=ax.transAxes,
                horizontalalignment='center', verticalalignment='bottom',
                fontsize=fontsize+2)
        if r==1 and 1==2: #!!!!!!
            plt.text(0.5, 1.45, pd[plotvar]['longname']+' '+pd[plotvar]['add'],
                transform=ax.transAxes,
                horizontalalignment='center', verticalalignment='bottom',
                fontsize=fontsize+3)
        print(r, '-'.join(runnames[::-1]))

        plt.text(-0.13, 1.0, '('+letters[r]+')',
            transform=ax.transAxes,
            horizontalalignment='left', verticalalignment='top',
            fontsize=fontsize-3)

        numformat = np.zeros((2), dtype='U10')
        for i in range(2):
            numformat[i] = '{:+1.2f}' if len(runnames) > 1  else '{:1.2f}'
            if abs(avg['-'.join(runnames[::-1])][plotvar][i]) < 1e-2:
                avg['-'.join(runnames[::-1])][plotvar][i] = 0.
                numformat[i] = '{:1.2f}'
        plt.text(0.18, 0.00, numformat[0].format(avg['-'.join(
            runnames[::-1])][plotvar][0])+pd[plotvar]['unit'][1:-1],
            transform=ax.transAxes, horizontalalignment='right',
            verticalalignment='bottom', fontsize=fontsize-5)
        plt.text(0.82, 0.00, numformat[1].format(avg['-'.join(
            runnames[::-1])][plotvar][1])+pd[plotvar]['unit'][1:-1],
            transform=ax.transAxes, horizontalalignment='left',
            verticalalignment='bottom', fontsize=fontsize-5)

    cax = fig.add_subplot(gs[1:-3,-1]) # (gs[-1,:-2]) for horizontal
    cbar = plt.colorbar(h1, ax=ax, cax=cax, orientation='vertical')
    cbar.set_label(pd[plotvar]['shortname']+' '+pd[plotvar]['unit'],
                   fontsize=fontsize+2)
    cbar.ax.tick_params(labelsize=fontsize+2)

plt.show(block=False)
